/**
 *   Licensed to the Apache Software Foundation (ASF) under one
 *   or more contributor license agreements.  See the NOTICE file
 *   distributed with this work for additional information
 *   regarding copyright ownership.  The ASF licenses this file
 *   to you under the Apache License, Version 2.0 (the
 *   "License"); you may not use this file except in compliance
 *   with the License.  You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */
package com.qlangtech.tis.plugin.ds;

import com.qlangtech.tis.offline.DbScope;
import com.qlangtech.tis.plugin.KeyedPluginStore;

import java.io.File;
import java.util.Objects;

/**
 * @author: baisui 百岁
 * @create: 2020-11-24 16:24
 */
public class DSKey extends KeyedPluginStore.Key<DataSourceFactory> {
    private final DbScope dbScope;

    public DSKey(String groupName, DbScope dbScope, String keyVal, Class<DataSourceFactory> pluginClass) {
        super(groupName, keyVal, pluginClass);
        this.dbScope = dbScope;
    }

    @Override
    protected String getSerializeFileName() {
        return groupName + File.separator + keyVal + File.separator + pluginClass.getName() + dbScope.getDBType();
    }

    public boolean isFacadeType() {
        return this.dbScope == DbScope.FACADE;
    }

    @Override
    public int hashCode() {
        return Objects.hash(this.keyVal, this.dbScope, pluginClass);
    }
}
